==> flowFlowJo_1.2.0.tar.gz: flowFlowJo/R/gatherFlowJoGates.R <==
setMethod("gatherFlowJoGates",
          signature(flowJoDOM = "XMLInternalDocument"), 
          definition = function(flowJoDOM, altFileLocation="") {

  # In FlowJos' current scheme, one dataSetNode will be the reference for a
  # single FCS file and all of it's gates.
  fjVersion     <- getFlowJoVersion(flowJoDOM);
  dataSetNodes  <- getNodeSet(flowJoDOM, "//Workspace/SampleList/Sample/DataSet");  
  resultsList   <- list(fcsName=vector(), 
                        filterName=vector(), 
                        filter=list(), 
                        terminalFilter=list(), 
                        fcsPath=vector());

  # The XML document may have paths to the FCS files that are designed for a 
  # a different operating system, and that may cause things to choke later on.
  # Also, sometimes the location of the referenced FCS files may be as a relative
  # path, which will create great problems later on.
  fsep         <- .Platform$file.sep;
  fcsPaths     <- unlist(lapply(dataSetNodes, xmlGetAttr, "uri"));
  fcsPaths     <- gsub("\\\\|\\/", fsep, fcsPaths, perl=TRUE);
#print(fcsPaths);
  foundFiles   <- file.exists(fcsPaths);  
  wspLocation  <- gsub("%20", " ", docName(flowJoDOM));

  # First try to use relative paths starting from the workspace location. 
  if ((sum(foundFiles) < 0.5 * length(foundFiles)) & 
      (nchar(altFileLocation) == 0)) {
     print("Trying a first approach to get file locations at: ");
     wspPath      <- dirname(wspLocation);
     testFCSPaths <- gsub("^\\.", wspPath, fcsPaths, perl=TRUE);
     cat(wspPath);  cat("\n");

     foundFiles   <- file.exists(testFCSPaths);
     if (sum(foundFiles) > 0.5 * length(foundFiles)) {
        fcsPaths  <- testFCSPaths;
     }
     print("Done with the first method");
  }

  # If that fails, try tacking on the info given in the altFileLocation
  if (sum(foundFiles) < 0.5 * length(foundFiles)) {
    print("Trying a second approach to get file locations at: ");
    altFileLocation  <- gsub("\\\\|\\/", fsep, altFileLocation);
    fcsPaths         <- paste(altFileLocation, basename(fcsPaths), sep=fsep);
    foundFiles       <- file.exists(fcsPaths);
    cat(altFileLocation);    cat("\n");
    print("Done with second approach...");
  }

  if (sum(file.exists(fcsPaths)) < 0.5 * length(foundFiles)) {
     cat("\nOK, I'm stumped, reading inside the workspace file, it thinks it resides at:\n");
     print(wspLocation);
     print("And inside the workspace file, it is looking for files at:");
     print(unique(dirname(unlist(lapply(dataSetNodes, xmlGetAttr, "uri")))));
     if (nchar(altFileLocation) > 0) {
        print("And you told me to try and also look for FCS files in:");
        print(altFileLocation);
     }
     stop("But I still can't find the FCS files referenced by this workspace.");
  }

  # Now iterate over each FCS file and pull out it's info.
  for (n in 1:length(dataSetNodes)) {
    fcsPath      <- fcsPaths[n];
    
    sampleId     <- xmlGetAttr(dataSetNodes[[n]], "sampleID");
    fcsName      <- basename(fcsPath);
#print("Uranium -- iterating over the dataSetNodes"); print(paste("  Working on", fcsName, " at ", fcsPath));
  
    # Back up one step, then find all gates hanging off of this FCS file
    parentSamp    <- xmlParent(dataSetNodes[[n]]);  
    polygonList   <- getNodeSet(parentSamp, ".//Population/PolygonGate");
    rectangleList <- getNodeSet(parentSamp, ".//Population/RectangleGate");
    rangeList     <- getNodeSet(parentSamp, ".//Population/RangeGate");
    gateList      <- c(rectangleList, rangeList, polygonList);    
    detailsList   <- list();

    # Accumulate information on each gate for this FCS file
    if (length(gateList) == 0) { next; }
    for (i in 1:length(gateList)) {
      gateNode     <- gateList[[i]];
      ancNodes     <- xmlAncestors(gateNode);
      ancs         <- sapply(ancNodes, xmlName); 
      
      # Typically, gates have a name such as "Lymphocytes". The next couple 
      # of lines catch those names.  If this gate is a nested gate then the 
      # code captures the names of those gates too.   
      index        <- ancs == "Population";
      parentNames  <- unlist(sapply(ancNodes, xmlGetAttr, "name")[index]);
      gateName     <- parentNames[ length(parentNames) ];            # name of this gate
      fullName     <- paste(c(fcsName, parentNames), collapse=":"); 
           
#cat("\n"); print(paste(n, i, fullName));      
      # Temporarily store this useful info for each gate.  Once I've got the info
      # for all the other gates for this FCS file, I'll be able to sort out and
      # concatenate (intersect?) each gate with his parents.  Note the key 
      # method called below -- 'gateNodeToFilterObject'
      tempList                 <- list();     
      tempList$filter          <- gateNodeToFilterObject(gateNode, quiet=TRUE,
                                                         stripComp=TRUE, version=fjVersion);
      tempList$filter@filterId <- paste(fcsName, gateName, sep=":");  
      tempList$ancNodes        <- ancNodes;  
      tempList$filterName      <- gateName;
      tempList$fullName        <- fullName;
      tempList$parentNames     <- parentNames;
      detailsList[[i]]         <- tempList;
    }
   
#print("Tantallum -- accumulated the detailsList"); print(str(detailsList));
      
    # Now go back and compare each gate to the others for this FCS file.
    # For each gate, I build up a list (parentalMatches) of which other
    # gates are the parents of this guy.
    for (i in 1:length(detailsList)) {
      parentalMatches = vector();
      for (j in 1:length(detailsList)) {
        if (j == i) next;
        lenCurrent = length(detailsList[[i]]$ancNodes);
        lenTest    = length(detailsList[[j]]$ancNodes);
        currName   = basename(detailsList[[i]]$fullName);
        testName   = basename(detailsList[[j]]$fullName);
#print("Niobium -- in the details list"); print(detailsList[[i]]$fullName);
        # Not sure if the "length" requirement is needed, but leave it in for now.
        # Vexingly, the following does not work!!
        #   (sum(detailsList[[j]]$ancNodes %in% detailsList[[i]]$ancNodes) 
        #          == (lenTest-1))) {
 
        if ((lenCurrent > lenTest) & (regexpr(testName, currName) == 1)) {           
          parentalMatches = c(parentalMatches, j);
        }            
      }
      # Finally, chain all the parental gates together for this terminal filter.
      # Boy this is hacked together very awkwardly!!!
      parentalMatches = sort(unique(parentalMatches));
      filterChain     = detailsList[[i]]$filter;
      if (length(parentalMatches) > 0) {
        filterChain = detailsList[[ parentalMatches[1] ]]$filter;
        for (k in 1:length(parentalMatches)) {
          if (k == 1) { next; }
          parentIndex = parentalMatches[k];
          filterChain = filterChain & detailsList[[ parentIndex ]]$filter;
        }
        filterChain = filterChain & detailsList[[i]]$filter;
      }
      filterChain@filterId = detailsList[[i]]$fullName;
      
#    c("fcsName", "filterName", "filter", "terminalFilter", "fcsPath");
      resultsList[[1]]  = c(resultsList[[1]], fcsName);
      resultsList[[2]]  = c(resultsList[[2]], detailsList[[i]]$fullName);
      resultsList[[3]]  = c(resultsList[[3]], filterChain);
      resultsList[[4]]  = c(resultsList[[4]], detailsList[[i]]$filter);
      resultsList[[5]]  = c(resultsList[[5]], fcsPath);         
  
      # Move on to the next gate for this FCS file
    }      
    # Move on to next FCS file
  }
  
  # Clean up the return list
  names(resultsList) <- c("fcsName", "filterName", "filter", 
                         "terminalFilter", "fcsPath"); 
  
  resultsList$fcsName       <- gsub("\\\\", .Platform$file.sep, resultsList$fcsName);
  resultsList$fcsPath       <- gsub("\\\\", .Platform$file.sep, resultsList$fcsPath);
  mode(resultsList$fcsName)    <- "character";
  mode(resultsList$filterName) <- "character";
  mode(resultsList$fcsPath)    <- "character";

  
  return(resultsList); 
});

==> flowFlowJo_1.4.0.tar.gz: flowFlowJo/R/gatherFlowJoGates.R <==
setMethod("gatherFlowJoGates",
          signature(flowJoDOM = "XMLInternalDocument"), 
          definition = function(flowJoDOM, altFileLocation="") {

  # In FlowJos' current scheme, one dataSetNode will be the reference for a
  # single FCS file and all of it's gates.
  fjVersion     <- getFlowJoVersion(flowJoDOM);
  dataSetNodes  <- getNodeSet(flowJoDOM, "//Workspace/SampleList/Sample/DataSet");  
  resultsList   <- list(fcsName=vector(), 
                        filterName=vector(), 
                        filter=list(), 
                        terminalFilter=list(), 
                        fcsPath=vector());

  # The XML document may have paths to the FCS files that are designed for a 
  # a different operating system, and that may cause things to choke later on.
  # Also, sometimes the location of the referenced FCS files may be as a relative
  # path, which will create great problems later on.
  fsep         <- .Platform$file.sep;
  fcsPaths     <- unlist(lapply(dataSetNodes, xmlGetAttr, "uri"));
  fcsPaths     <- gsub("\\\\|\\/", fsep, fcsPaths, perl=TRUE);
  foundFiles   <- file.exists(fcsPaths);  
  wspLocation  <- gsub("%20", " ", docName(flowJoDOM));
  wspLocation  <- gsub("^file:\\/*", "", wspLocation)
  
  # First try to use relative paths starting from the workspace location. 
  if ((sum(foundFiles) < 0.5 * length(foundFiles)) & 
      (nchar(altFileLocation) == 0)) {
     wspPath      <- dirname(wspLocation);
     testFCSPaths <- paste(wspPath, basename(fcsPaths), sep=fsep);
     foundFiles   <- file.exists(testFCSPaths);
     
     cat("\nHaving a bit of trouble finding the FCS files, \nWill try: ", wspPath);
     if (sum(foundFiles) > 0.5 * length(foundFiles)) {
      fcsPaths  <- testFCSPaths;
      cat(" .... found them!\n")
     } else {
      cat(" .... hmmmm, let me try something different\n");
     }     
  }

  # If that fails, try tacking on the info given in the altFileLocation
  if (sum(foundFiles) < 0.5 * length(foundFiles)) {
    altFileLocation  <- gsub("\\\\|\\/", fsep, altFileLocation);
    testFCSPaths     <- paste(altFileLocation, basename(fcsPaths), sep=fsep);
    foundFiles       <- file.exists(testFCSPaths);
    cat("Having a bit of trouble finding your FCS files, \nI'll look in ", altFileLocation);   
    if (sum(foundFiles) > 0.5 * length(foundFiles)) {
      fcsPaths  <- testFCSPaths;
      cat(".... found them!\n")
    } else {
      cat(" .... no luck\n");
      stop("I'm sorry, I can't locate the FCS files referred to in your FlowJo workspace.")
    }     
  }


  # Now iterate over each FCS file and pull out it's info.
  for (n in 1:length(dataSetNodes)) {
    fcsPath      <- fcsPaths[n];  
    sampleId     <- xmlGetAttr(dataSetNodes[[n]], "sampleID");
    fcsName      <- basename(fcsPath);
    cat("Working on: ", fcsName)

    # Back up one step, then find all gates hanging off of this FCS file
    ns = c(gating="http://www.isac-net.org/std/Gating-ML/v1.5/gating")
    parentSamp    <- xmlParent(dataSetNodes[[n]]); 
    
    polygonPath   = ".//Population/Gate/gating:PolygonGate"
    rectanglePath = ".//Population/Gate/gating:RectangleGate"
    rangePath     = ".//Population/Gate/gating:RangeGate"
    if (compareVersion(fjVersion, "7.4.9") < 0) {
      polygonPath   = ".//Population/PolygonGate"
      rectanglePath = ".//Population/RectangleGate"
      rangePath     = ".//Population/RangeGate"
    }

    polygonList   <- getNodeSet(parentSamp, polygonPath,   namespaces=ns);
    rectangleList <- getNodeSet(parentSamp, rectanglePath, namespaces=ns);
    rangeList     <- getNodeSet(parentSamp, rangePath,     namespaces=ns);
    gateList      <- c(rectangleList, rangeList, polygonList);  
    detailsList   <- list();

    # Accumulate information on each gate for this FCS file
    if (length(gateList) == 0) { next; cat("\n"); }
    for (i in 1:length(gateList)) {
      gateNode     <- gateList[[i]];
      ancNodes     <- xmlAncestors(gateNode);
      ancs         <- sapply(ancNodes, xmlName); 
      
      # Typically, gates have a name such as "Lymphocytes". The next couple 
      # of lines catch those names.  If this gate is a nested gate then the 
      # code captures the names of those gates too.   
      index        <- ancs == "Population";
      parentNames  <- unlist(sapply(ancNodes, xmlGetAttr, "name")[index]);
      gateName     <- parentNames[ length(parentNames) ];            # name of this gate
      fullName     <- paste(c(fcsName, parentNames), collapse=":"); 
      cat(" ", paste(parentNames, collapse=":"))
                 
      # Temporarily store this useful info for each gate.  Once I've got the info
      # for all the other gates for this FCS file, I'll be able to sort out and
      # concatenate (intersect?) each gate with his parents.  Note the key 
      # method called below -- 'gateNodeToFilterObject'
      tempList                 <- list();     
      tempList$filter          <- gateNodeToFilterObject(gateNode, quiet=TRUE, stripComp=TRUE);
      tempList$filter@filterId <- paste(fcsName, gateName, sep=":");  
      tempList$ancNodes        <- ancNodes;  
      tempList$filterName      <- gateName;
      tempList$fullName        <- fullName;
      tempList$parentNames     <- parentNames;
      detailsList[[i]]         <- tempList;
    }
    cat("\n");
      
    # Now go back and compare each gate to the others for this FCS file.
    # For each gate, I build up a list (parentalMatches) of which other
    # gates are the parents of this guy.
    for (i in 1:length(detailsList)) {
      parentalMatches = vector();
      for (j in 1:length(detailsList)) {
        if (j == i) next;
        lenCurrent = length(detailsList[[i]]$ancNodes);
        lenTest    = length(detailsList[[j]]$ancNodes);
        currName   = basename(detailsList[[i]]$fullName);
        testName   = basename(detailsList[[j]]$fullName);
        # Not sure if the "length" requirement is needed, but leave it in for now.
        # Vexingly, the following does not work!!
        #   (sum(detailsList[[j]]$ancNodes %in% detailsList[[i]]$ancNodes) 
        #          == (lenTest-1))) {
 
        if ((lenCurrent > lenTest) & (regexpr(testName, currName) == 1)) {           
          parentalMatches = c(parentalMatches, j);
        }            
      }
      # Finally, chain all the parental gates together for this terminal filter.
      # Boy this is hacked together very awkwardly!!!
      parentalMatches = sort(unique(parentalMatches));
      filterChain     = detailsList[[i]]$filter;
      if (length(parentalMatches) > 0) {
        filterChain = detailsList[[ parentalMatches[1] ]]$filter;
        for (k in 1:length(parentalMatches)) {
          if (k == 1) { next; }
          parentIndex = parentalMatches[k];
          filterChain = filterChain & detailsList[[ parentIndex ]]$filter;
        }
        filterChain = filterChain & detailsList[[i]]$filter;
      }
      filterChain@filterId = detailsList[[i]]$fullName;
      
#    c("fcsName", "filterName", "filter", "terminalFilter", "fcsPath");
      resultsList[[1]]  = c(resultsList[[1]], fcsName);
      resultsList[[2]]  = c(resultsList[[2]], detailsList[[i]]$fullName);
      resultsList[[3]]  = c(resultsList[[3]], filterChain);
      resultsList[[4]]  = c(resultsList[[4]], detailsList[[i]]$filter);
      resultsList[[5]]  = c(resultsList[[5]], fcsPath);         
  
      # Move on to the next gate for this FCS file
    }      
    # Move on to next FCS file
  }
  
  # Clean up the return list
  names(resultsList) <- c("fcsName", "filterName", "filter", 
                         "terminalFilter", "fcsPath"); 
  
  resultsList$fcsName       <- gsub("\\\\", .Platform$file.sep, resultsList$fcsName);
  resultsList$fcsPath       <- gsub("\\\\", .Platform$file.sep, resultsList$fcsPath);
  mode(resultsList$fcsName)    <- "character";
  mode(resultsList$filterName) <- "character";
  mode(resultsList$fcsPath)    <- "character";

  
  return(resultsList); 
});
